@@ -1,7 +1,7 @@
 package Stor;
 use v5.20;
 
-our $VERSION = '1.0.0';
+our $VERSION = '1.1.3';
 
 use Mojo::Base -base, -signatures;
 use Syntax::Keyword::Try;
@@ -108,16 +108,23 @@ sub get_from_s3 ($self, $c, $sha) {
     )->http_request;
 
     # build Mojo request inside transaction for proper streaming
+    $c->app->ua->max_response_size(0);
     my $tx = $c->app->ua->build_tx(GET => $http_request->uri->as_string);
     for my $header_key ('authorization', 'date') {
         $tx->req->headers->header($header_key => $http_request->headers->header($header_key));
     }
 
-    $tx->res->content->on(
+    $tx->res->content->unsubscribe('read')->on(
         read => sub {
             my (undef, $chunk) = @_;
             if ($chunk) {
-                $c->write($chunk);
+                try {
+                    $c->write($chunk);
+                }
+                catch{
+                    $c->app->log->warning("Writing chunk failed: $@");
+                    $tx->res->content->unsubscribe('read');
+                }
             }
         }
     );
@@ -138,6 +145,9 @@ sub get_from_s3 ($self, $c, $sha) {
 
 sub get ($self, $c) {
     my $sha = $c->param('sha');
+
+    $self->statsite->increment('request.get.count');
+
     try {
         failure::stor::filenotfound->throw({
             msg     => "Given hash '$sha' isn't SHA256",
@@ -151,7 +161,6 @@ sub get ($self, $c) {
         }
     }
     catch {
-        $c->app->log->debug("$@");
         if ($@->$_isa('failure::stor::filenotfound')) {
             $c->render(status => 404, text => "$@");
         }
@@ -159,10 +168,13 @@ sub get ($self, $c) {
             $self->statsite->increment('error.get.500.count');
             $c->render(status => 500, text => "$@");
         }
-    }
-    finally {
+
         if ($@->$_isa('failure::stor')) {
             $self->statsite->increment($@->payload->{statsite_key});
+            $c->app->log->warning($@->msg);
+        }
+        else {
+            $c->app->log->error("$@");
         }
     }
 }
@@ -170,6 +182,8 @@ sub get ($self, $c) {
 sub post ($self, $c) {
     my $sha  = $c->param('sha');
 
+    $self->statsite->increment('request.post.count');
+
     if ($sha !~ /^[A-Fa-f0-9]{64}$/) {
         $self->statsite->increment('error.post.malformed_sha.count');
         $c->render(status => 412, text => "Given hash '$sha' isn't sha256");
@@ -193,8 +207,7 @@ sub post ($self, $c) {
     my $content_sha = sha256_hex($file->slurp());
     if (lc($sha) ne lc($content_sha)) {
         $self->statsite->increment('error.post.bad_sha.count');
-        $c->render(status => 412, text =>
-            "Content sha256 $content_sha doesn't match given sha256 $sha");
+        $self->_render_and_log($c, 412, "Content sha256 $content_sha doesn't match given sha256 $sha");
         return
     }
 
@@ -207,11 +220,11 @@ sub post ($self, $c) {
     catch {
         if ($@->$_isa('failure::stor')) {
             $self->statsite->increment($@->payload->{statsite_key});
-            $c->render(status => 507, text => "$@");
+            $self->_render_and_log($c, 507, "$@");
             return
         }
         $self->statsite->increment('error.post.unknown.count');
-        $c->render(status => 500, text => "$@");
+        $self->_render_and_log($c, 500, "$@");
         return
     }
 }
@@ -275,6 +288,11 @@ sub save_file ($self, $file, $sha, $storage_pair) {
     return \@all_paths;
 }
 
+sub _render_and_log($self, $c, $status, $text) {
+    $c->render(status => $status, text => $text);
+    $c->app->log->warning("$status $text");
+}
+
 sub _lookup ($self, $sha, $return_all_paths = '') {
     my @paths;
     my $attempt = 0;
@@ -311,20 +329,21 @@ sub _sha_to_filepath($self, $sha) {
 
 sub _stream_found_file($self, $c, $path) {
     my $fh = $path->openr_raw();
+    my $time = time;
+    my $total_size = 0;
     my $drain; $drain = sub {
         my ($c) = @_;
 
         my $chunk;
-        my $tm_chunk = time;
         my $size = read($fh, $chunk, 1024 * 1024);
+        $total_size += $size;
         if (!$size) {
             close($fh);
             $drain = undef;
+            $self->statsite->update('success.get.ok_old.size', $total_size);
+            $self->statsite->timing('success.get.ok_old.time', (time - $time) * 1000);
         }
-
         $c->write($chunk, $drain);
-        $self->statsite->update('success.get.ok_old.size', $size);
-        $self->statsite->timing('success.get.ok_old.time', (time - $tm_chunk) * 1000);
     };
     $c->$drain;
 }
@@ -385,11 +404,11 @@ we prefer L<hypnotoad|https://mojolicious.org/perldoc/Mojo/Server/Hypnotoad> ser
 =head2 configuration example
 
     {
-        "statsite": {                                                                 
-            "host": "STATSITE_HOST",                                       
+        "statsite": {
+            "host": "STATSITE_HOST",
             "prefix": "stor.dev",
             "sample_rate": 0.1
-        },  
+        },
         "storage_pairs": [
             ["/mnt/data1", "/mnt/data2"],
             ["/mnt/data3", "/mnt/data4"]
@@ -404,7 +423,7 @@ we prefer L<hypnotoad|https://mojolicious.org/perldoc/Mojo/Server/Hypnotoad> ser
         "memcached_servers": ["MEMCACHED_SERVER1"],
         "secret": "https://mojolicious.org/perldoc/Mojolicious/Guides/FAQ#What-does-Your-secret-passphrase-needs-to-be-changed-mean",
         "basic_auth": "writer:writer_pass"
-    } 
+    }
 
 =head2 Service Responsibility
 
